BASE_URL = 'https://backend.segna.io/public/server-side'


class API:
    RUN_JOB = '/v1/run-job'
    JOB_DATA = '/v1/job-data'


class Request:
    DATA_TYPE = 'dataType'

    class DataTypeFormats:
        DATAFRAME = 'dataframe'


class Response:
    JOB_ID = 'jobId'
    DATA = 'data'
    ERROR = 'errorMessage'
    DATA_TYPES = 'dtypes'




import pandas as pd
import requests

import segna.constants as cst
from segna.job_config import JobConfig

SEGNA_ACCESS_KEY = None
SEGNA_SECRET_ACCESS_KEY = None


def init(access_key, secret_key):
    global SEGNA_ACCESS_KEY, SEGNA_SECRET_ACCESS_KEY
    SEGNA_ACCESS_KEY = access_key
    SEGNA_SECRET_ACCESS_KEY = secret_key


def configure_job(pipeline_id, job_name='temp_job') -> JobConfig:
    return JobConfig(pipeline_id, job_name)


def run_job(config_obj) -> str:
    if SEGNA_ACCESS_KEY is None or SEGNA_SECRET_ACCESS_KEY is None:
        raise ValueError('Please specify your access and secret key by calling segna.init')
    r = requests.post(cst.BASE_URL + cst.API.RUN_JOB,
                      json=config_obj.to_json(),
                      auth=(SEGNA_ACCESS_KEY, SEGNA_SECRET_ACCESS_KEY))
    r.raise_for_status()
    job_id = r.json()[cst.Response.JOB_ID]
    return job_id


def get_job_data(job_id) -> pd.DataFrame:
    payload = {cst.Request.DATA_TYPE: cst.Request.DataTypeFormats.DATAFRAME}
    r = requests.post(cst.BASE_URL + cst.API.JOB_DATA + f'/{job_id}',
                      auth=(SEGNA_ACCESS_KEY, SEGNA_SECRET_ACCESS_KEY),
                      params=payload)
    response_data = r.json()
    try:
        r.raise_for_status()
    finally:
        error_message = response_data.get(cst.Response.ERROR)
        if error_message:
            print("ERROR: " + error_message)

    list_data = response_data[cst.Response.DATA]
    dtypes = response_data[cst.Response.DATA_TYPES]
    columns = list_data.pop(0)
    data = pd.DataFrame(list_data, columns=columns)
    data = data.astype(dtypes)
    return data



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name="segna",
    version="1.0.2",
    description="Run Segna pipelines from your Python code!",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://segna.io",

    author="Segna",
    author_email="josh@segna.io",
    license="MIT",
    keywords=['Segna', 'API'],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        'Intended Audience :: Developers',
        "Programming Language :: Python :: 3",
    ],
    packages=["segna"],
    include_package_data=True,
    install_requires=['requests', 'datetime', 'pandas']
)


import re
import datetime

DATE_RANGE_FILTER = 'date'
TEXT_FILTER = 'text'
NUMBER_RANGE_FILTER = 'number'
FILTER_TYPES = [DATE_RANGE_FILTER, TEXT_FILTER, NUMBER_RANGE_FILTER]

# Check for date in format YYYY-XX-XX
DATE_CHECK = re.compile(r'^\d\d\d\d[-/]\d\d[-/]\d\d$')


def date_format_checker(date):
    if not DATE_CHECK.match(date):
        return False

    year = int(date[:4])
    month = int(date[5:7])
    day = int(date[8:10])
    # Check for date in format YYYY-MM-DD
    # TODO: allow for YYYY-M-D dates without 0 padding
    try:
        d = datetime.datetime(year,month,day)
        return True
    except ValueError:
        return False
    

class JobConfig:
    def __init__(self, pipeline_id, job_name):
        self.pipeline_id = pipeline_id
        self.filters = {}
        self.__allow_push_to_db = False
        self.__job_name = job_name

    def add_filter(self, field, filter_type, value, negate=False):
        if filter_type not in FILTER_TYPES:
            raise ValueError('"filter_type" should be one of ' + str(FILTER_TYPES))
        if filter_type == TEXT_FILTER:
            if not isinstance(value, str):
                raise ValueError('Value must be a string when using a text filter.')
            filter_params = {'filter_type': filter_type, 'value': value, 'not': negate}

        elif filter_type in [DATE_RANGE_FILTER, NUMBER_RANGE_FILTER]:
            if not isinstance(value, list) or len(value) != 2:
                raise ValueError('Value must be a list of length 2 [start, end] when using a date or number range filter.')
            if filter_type == NUMBER_RANGE_FILTER:
                if not all([isinstance(v, int) or isinstance(v, float) for v in value]):
                    raise ValueError('Each entry of the filter value must be numeric when using a number range filter.')
            elif filter_type == DATE_RANGE_FILTER:
                if not all([date_format_checker(v) for v in value]):
                    raise ValueError('Please specify a start and an end date in "YYYY-MM-DD" format.')
            
            filter_params = {'filter_type': filter_type, 'start': value[0], 'end': value[1], 'not': negate}

        self.filters[field] = filter_params
        
    def get_push_permissions(self):
        return self.__allow_push_to_db

    def enable_push_to_db(self, warn=True):
        if warn:
            print('WARNING: this will enable pushing of pipeline output to connected databases.')
        self.__allow_push_to_db = True
    
    def disable_push_to_db(self):
        self.__allow_push_to_db = False

    def clear_filters(self):
        self.filters = {}

    def to_json(self):
        return {
            'job_name': self.__job_name,
            'pipelineID': self.pipeline_id,
            'allowPushToDB': self.__allow_push_to_db,
            'filters': self.filters
        }
